import imaplib

servers = {
    "Gmail": "imap.gmail.com",
    "Outlook": "imap-mail.outlook.com",
    "Hotmail": "imap-mail.outlook.com",
    "Icloud": "imap.mail.me.com",
}


def mail_selection(mail, selection, custom_address="n/a"):
    """Searches mail box for selected emails

    Args:
        mail (imaplib connection): imaplib's connection to the email object
        selection (string): type of emails to be marked for delition
                                       passes the address . Defaults to "n/a".
    Returns:
        messages : returns list of messages that fit the search
    """
    messages = None

    if selection == "Newsletters":
        status, messages = mail.search(None, '(BODY "unsubscribe")')

    elif selection == "All":
        status, messages = mail.search(None, "ALL")

    elif selection == "Custom":
        status, messages = mail.search(None, f'FROM "{custom_address}"')

    if not messages or messages[0] == b"":
        print("No emails found matching that criteria.")

    return messages[0].split()


def connect_to_email(provider, username, password, selection, custom_address="n/a"):
    """Tries connection to email server ,
    checks connection and marks emails for deletion
    """
    mail = imaplib.IMAP4_SSL(servers[provider])

    try:
        print(f"Logging in as {username}...")
        mail.login(username, password)
        mail.select("INBOX")

        email_ids = mail_selection(mail, selection, custom_address)
        print(f"Found {len(email_ids)} emails.")
        if len(email_ids) == 0:
            return

        # loops through the list in chunks of 100
        for i in range(0, len(email_ids), 100):
            batch = email_ids[i : i + 100]

            #  allows sending one command for 100 emails
            email_id_string = b",".join(batch)

            mail.store(email_id_string, "+FLAGS", "\\Deleted")

            # deletes the batch immediately
            mail.expunge()

            print(
                f"Deleted {min(i + 100, len(email_ids))} emails out of {len(email_ids)}"
            )

        mail.close()
        mail.logout()
        print("Deletion complete.")

    except Exception as error:
        print(f"Error: {error}")
        print("Please run 'setup' again")



import imaplib
from collections import defaultdict
import email
from bs4 import BeautifulSoup
import re
import os

servers = {
    "Gmail": "imap.gmail.com",
    "Outlook": "imap-mail.outlook.com",
    "Hotmail": "imap-mail.outlook.com",
    "Icloud": "imap.mail.me.com",
}


def start_unsubscriber(provider, username, password):
    """Establishes connection and logs in."""
    mail = imaplib.IMAP4_SSL(servers[provider])
    mail.login(username, password)
    mail.select("inbox")
    return mail


def extract_links_from_html(html_content):
    """Parses HTML and finds all 'unsubscribe' links."""
    soup = BeautifulSoup(html_content, "html.parser")
    links = [
        link["href"]
        for link in soup.find_all("a", href=True)
        if "unsubscribe" in link["href"].lower()
    ]
    return links


def process_email_data(raw_email_bytes):
    """Helper function to parse a single email bytes object and return a dictionary of {sender: set(links)}."""
    msg = email.message_from_bytes(raw_email_bytes)
    # extract sender for readability
    sender_header = msg.get("From", "Unknown Sender")
    # clean up the sender string
    match = re.search(r"<(.*?)>", sender_header)
    sender = match.group(1) if match else sender_header.strip()
    email_links = defaultdict(set)
    # extract links
    if msg.is_multipart():
        for part in msg.walk():
            if part.get_content_type() == "text/html":
                try:
                    html_content = part.get_payload(decode=True).decode(errors="ignore")
                    links = extract_links_from_html(html_content)
                    for link in links:
                        email_links[sender].add(link)
                except Exception:
                    pass
    else:
        content_type = msg.get_content_type()
        if content_type == "text/html":
            try:
                content = msg.get_payload(decode=True).decode(errors="ignore")
                links = extract_links_from_html(content)
                for link in links:
                    email_links[sender].add(link)
            except Exception:
                pass
    return email_links


def save_links(links_with_sender):
    """Saves the list of extracted links to a text file and opens it in the browser."""
    # Save to links.txt
    with open("links.txt", "w") as f:
        for sender, link in links_with_sender:
            f.write(f"{sender}: {link}\n")

    # Get absolute path
    txt_file_path = os.path.abspath("links.txt")
    print(f"File saved at: {txt_file_path}")


def search_for_email(provider, username, password):
    """
    Main function to connect, search, fetch, process emails, and save links.
    The automated 'clicking' functionality has been removed.
    """
    mail = start_unsubscriber(provider, username, password)
    print("Searching for emails...")
    print("Process could take a few minutes")
    _, search_data = mail.search(None, '(BODY "unsubscribe")')
    email_ids = search_data[0].split()
    print(f"Found {len(email_ids)} emails. Fetching content...")
    all_links_dict = defaultdict(set)
    batch_size = 50
    for i in range(0, len(email_ids), batch_size):
        batch_ids = email_ids[i : i + batch_size]
        id_string = b",".join(batch_ids)
        status, response_data = mail.fetch(id_string, "(RFC822)")
        for response_part in response_data:
            if isinstance(response_part, tuple):
                links_dict = process_email_data(response_part[1])
                for sender, links in links_dict.items():
                    all_links_dict[sender].update(links)
    mail.logout()
    # convert to tuples for saving to the file
    links_with_sender = []
    for sender, links in all_links_dict.items():
        for link in links:
            links_with_sender.append((sender, link))
    print(f"Found {len(links_with_sender)} unique subscriptions/links:")
    # Function call: save_links()
    save_links(links_with_sender)



import os

import click

from .clean import connect_to_email

from .unsubscribe import search_for_email

ENV_FILE = "setup_op.env"


@click.group()
def astria():
    pass


@astria.command()
@click.option(
    "--provider",
    prompt="Enter your email provider",
    type=click.Choice(["Gmail", "Outlook", "Icloud", "Hotmail"], case_sensitive=False),
)
@click.option("--address", prompt="Enter your email address")
@click.option("--password", prompt="Enter your app password", hide_input=True)
def setup(provider, address, password):
    """Creates local configuration file."""
    try:
        with open(ENV_FILE, "w") as f:
            f.write(f"PROVIDER={provider}\n")
            f.write(f"ADDRESS={address}\n")
            f.write(f"PASSWORD={password}\n")
        click.echo("Configuration saved!")
    except Exception as e:
        click.echo(f"Error saving file: {e}")


@astria.command()
def unsubscribe():
    """Saves all the "unsubscribe" links to "links.txt" """
    creds = {}

    if os.path.exists(ENV_FILE):
        click.echo("Loading credentials from file...")
        with open(ENV_FILE, "r") as f:
            for line in f:
                if "=" in line:
                    key, value = line.strip().split("=", 1)
                    creds[key] = value

        if "ADDRESS" in creds and "PASSWORD" in creds:
            search_for_email(
                creds["PROVIDER"],
                creds["ADDRESS"],
                creds["PASSWORD"],
            )
        else:
            click.echo(
                "Error: Missing ADDRESS or PASSWORD in setup file. Run 'setup' again."
            )

    else:
        click.echo("Error: No credentials found please run 'setup' command first")


@astria.command()
@click.option(
    "--selection",
    prompt="Enter the type of mails you want to delete",
    type=click.Choice(["Newsletters", "Custom", "All"], case_sensitive=False),
)
def clean_email(selection):
    """Connects and cleans
    email using saved credentials."""
    address = ""
    if selection == "Custom":
        address = input("Enter the specific email address you want to delete: ")

    creds = {}

    if os.path.exists(ENV_FILE):
        click.echo("Loading credentials from file...")
        with open(ENV_FILE, "r") as f:
            for line in f:
                if "=" in line:
                    key, value = line.strip().split("=", 1)
                    creds[key] = value

        if "ADDRESS" in creds and "PASSWORD" in creds:
            connect_to_email(
                creds["PROVIDER"],
                creds["ADDRESS"],
                creds["PASSWORD"],
                selection,
                address,
            )
        else:
            click.echo(
                "Error: Missing ADDRESS or PASSWORD in setup file. Run 'setup' again."
            )

    else:
        click.echo("Error: No credentials found please run 'setup' command first")


if __name__ == "__main__":
    astria()



from setuptools import setup, find_packages

setup(
    name="astria",
    version="0.1",
    description="Astria is an email cleanup CLI tool that allows for fast deletion and newsletter unsubscribing.",
    author="Vallg-333",
    author_email="vallg8343@proton.me",
    packages=find_packages(),
    install_requires=["click", "bs4"],
    entry_points={
        "console_scripts": ["astria=astria.cli:astria"],
    },
)
